--------------------------------------------------------------------------------
-- Copyright (c) 2003 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2Se Application Note: Game of Life
-- File:        VGA.vhd
-- Description: VGA interface.
-- History:     2003jan09, FB, created.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity vga is
	port (
		-- clock & reset
		clk25        : in  std_logic;
		rst          : in  std_logic;
		-- position info
		row          : out std_logic_vector(9 downto 0);
		col          : out std_logic_vector(9 downto 0);
		hsync        : out std_logic;
		vsync        : out std_logic;
		hblk         : out std_logic;
		vblk         : out std_logic;
		-- video in
		pel          : in  std_logic;
		-- video out
		r            : out std_logic;
		g            : out std_logic;
		b            : out std_logic;
		rb           : out std_logic;
		gb           : out std_logic;
		bb           : out std_logic);
end vga;

--------------------------------------------------------------------------------
architecture bhv of vga is
	-- vga timing
	signal   hcnt, vcnt:      unsigned(9 downto 0);         -- horizontal/ vertical counters
	signal   crow, ccol:      unsigned(3 downto 0);         -- row/col within char
	signal   trow, tcol:      unsigned(5 downto 0);         -- text row/ col
	signal   hbki, vbki, blk: std_logic;                    -- blanking
	-- delayed signals
	signal   crow1, ccol1:    unsigned(crow'range);
	signal   blk1, blk2:      std_logic;
	--
	--        |__________a     d___          |___________e    h___
	-- rgb   _/__________\_____/___   lines _////////////_____////
	--        |                              |
	--       _|____________b c_____         _|____________f g_____
	-- hsync  |            \_/        vsync  |            \_/
	--
	--        d   = 31.77us                  h   = 16.6ms
	--        c-b =  3.77us                  g-f = 64us
	--        d-c =  1.89us                  h-g =  1.02ms
	--        a   = 25.17us                  e   = 15.25ms
	--        b-a =  0.94us                  f-e =  0.35ms
	-- 
	-- Mode       clock       h         v      a    b    c    d      e   f   g   h
	-- 640x480    25.175MHz   31.5kHz   60Hz   640  664  760  800    480 491 493 525
	-- 640x480    24MHz       31.5kHz   60Hz   640  645  730  762    480 491 493 525 (c-b=90, d-c=45)
	-- 800x600    36MHz       35.1kHz   --     800  824  896  1024   600 601 603 625
	-- 1024x768   65MHz       48.4kHz   --     1024 1032 1176 1344   768 771 777 806
	--
	constant hcnt_a: UNSIGNED(hcnt'range):= TO_UNSIGNED(639, hcnt'length); -- blanking start
	constant hcnt_b: UNSIGNED(hcnt'range):= TO_UNSIGNED(644, hcnt'length); -- h-sync start
	constant hcnt_c: UNSIGNED(hcnt'range):= TO_UNSIGNED(729, hcnt'length); -- h-sync end
	constant hcnt_d: UNSIGNED(hcnt'range):= TO_UNSIGNED(761, hcnt'length); -- h length
	constant vcnt_e: UNSIGNED(vcnt'range):= TO_UNSIGNED(479, vcnt'length); -- v blanking
	constant vcnt_f: UNSIGNED(vcnt'range):= TO_UNSIGNED(490, vcnt'length); -- v-sync start
	constant vcnt_g: UNSIGNED(vcnt'range):= TO_UNSIGNED(492, vcnt'length); -- v-sync end
	constant vcnt_h: UNSIGNED(vcnt'range):= TO_UNSIGNED(524, vcnt'length); -- v length
	constant hcnt_v: UNSIGNED(hcnt'range):= TO_UNSIGNED(699, hcnt'length); -- next line
begin
	
	------------------------------------
	-- horizontal timing
	process(clk25, rst)
	begin
		if rst= '1' then
			-- synopsys translate_off
			hcnt<= hcnt_d; --TO_UNSIGNED(0, hcnt'length);
			hbki<= '0';
			ccol<= TO_UNSIGNED(0, ccol'length);
			tcol<= TO_UNSIGNED(0, tcol'length);
			hsync<= '1';
			-- synopsys translate_on
		elsif rising_edge(clk25) then
			if hcnt= hcnt_d then 
				hcnt<= TO_UNSIGNED(0, hcnt'length);
				hbki<= '0';
				
				ccol<= TO_UNSIGNED(0, ccol'length);
				tcol<= TO_UNSIGNED(0, tcol'length);
			else
				hcnt<= hcnt + 1;
				
				if ccol= TO_UNSIGNED(9, ccol'length) then
					ccol<= TO_UNSIGNED(0, ccol'length);
					tcol<= tcol + 1;
				else
					ccol<= ccol + 1;
				end if;
			end if;
			
			if hcnt= hcnt_a  then hbki <= '1'; end if; -- blanking on
			if hcnt= hcnt_b  then hsync<= '0'; end if; -- h-sync start
			if hcnt= hcnt_c  then hsync<= '1'; end if; -- h-sync end
		end if;
	end process;
	
	------------------------------------
	-- vertical timing
	process(clk25, rst)
	begin
		if rst= '1' then
			-- synopsys translate_off
			vcnt<= vcnt_h; --TO_UNSIGNED(0, vcnt'length);
			vbki<= '0';
			crow<= TO_UNSIGNED(0,  crow'length);
			trow<= TO_UNSIGNED(56, trow'length);
			--			trow<= TO_UNSIGNED(0,  trow'length);
			vsync<= '1';
			-- synopsys translate_on
		elsif rising_edge(clk25) then
			if hcnt= hcnt_v then
				if vcnt= vcnt_h then
					vcnt<= TO_UNSIGNED(0, vcnt'length);
					vbki<= '0';
					
					crow<= TO_UNSIGNED(0,  crow'length);
					trow<= TO_UNSIGNED(56, trow'length);
					--					trow<= TO_UNSIGNED(0,  trow'length);
				else
					vcnt<= vcnt + 1;
					
					if crow= TO_UNSIGNED(9, crow'length) then
						crow<= TO_UNSIGNED(0, crow'length);
						trow<= trow + 1;
					else
						crow<= crow + 1;
					end if;
				end if;
			end if;
			
			if vcnt= vcnt_e then vbki <= '1'; end if; -- blanking on
			if vcnt= vcnt_f then vsync<= '0'; end if; -- v-sync start
			if vcnt= vcnt_g then vsync<= '1'; end if; -- v-sync end
		end if;
	end process;
	
	------------------------------------
	-- output signals
	blk  <= hbki or vbki;
	row  <= std_logic_vector(vcnt);
	col  <= std_logic_vector(hcnt);
	hblk <= hbki;
	vblk <= vbki;
	r    <= '0' when blk= '1' else pel;	
	g    <= '0' when blk= '1' else pel;	
	b    <= '0' when blk= '1' else '1';	
	rb   <= '0' when blk= '1' else pel;	
	gb   <= '0' when blk= '1' else pel;	
	bb   <= '0' when blk= '1' else '1';	
end bhv;

--------------------------------------------------------------------------------
-- end of file

